<pre>
$_GET = <?php var_export($_GET); ?>

$_POST = <?php var_export($_POST); ?>

$_REQUEST = <?php var_export($_REQUEST); ?>

$_SERVER = <?php var_export($_SERVER); ?>

$_ENV = <?php var_export($_ENV); ?>

$_COOKIE = <?php var_export($_COOKIE); ?>

$_FILES = <?php var_export($_FILES); ?>
</pre>
