<pre>
<?php

global $g1,$g2,$g3,$g4,$g5;

if (isset($g1)) {
   echo "g1=$g1\n";
} else {
   echo "g1 not set\n";
}

if (isset($g2)) {
   echo "g2=$g2\n";
} else {
   echo "g2 not set\n";
}

if (isset($g3)) {
   echo "g3=$g3\n";
} else {
   echo "g3 not set\n";
}

if (isset($g4)) {
   echo "g4=$g4\n";
} else {
   echo "g4 not set\n";
}

if (isset($g5)) {
   echo "g5=$g5\n";
} else {
   echo "g5 not set\n";
}

?>
</pre>